import useUserStore from '@/stores/user'

/**
 * 根据传入的指令值数组判断是否具有该权限，根据权限决定是否显示功能按钮
 * @param el 指令绑定元素
 * @param binding 一些绑定数据
 */
function checkPermission(el: any, binding: any) {
  // 获取绑定的值，此处为权限
  const { value } = binding
  // 获取所有的功能指令
  const userStore = useUserStore()
  const points = userStore.userInfo?.permission?.points
  // 当传入的指令集为数组时
  if (value && value instanceof Array) {
    // 匹配对应的指令
    // some 表示有一个返回 true，则 some 函数返回 true
    const hasPermission = points?.some((point) => {
      return value.includes(point)
    })
    // 如果无法匹配，则表示当前用户无该指令，那么删除对应的功能按钮
    if (!hasPermission) {
      // el.parentNode && el.parentNode.removeChild(el)
      el.remove()
    }
  } else {
    // 指令值为非数组
    throw new Error(
      'v-permission value should be a Array, such as  ["admin", "editor"]'
    )
  }
}

export default {
  // 在绑定元素的父组件被挂载后调用
  mounted(el: any, binding: any) {
    checkPermission(el, binding)
  },
  // 在包含组件的 VNode 及其子组件的 VNode 更新后调用
  update(el: any, binding: any) {
    checkPermission(el, binding)
  }
}
