import { createRouter, createWebHistory } from 'vue-router'
import type { RouteRecordRaw } from 'vue-router'
import useUserStore from '@/stores/user'
import Layout from '@/layout/layout.vue'
import UserManage from './modules/user-manage'
import RoleList from './modules/role-list'
import PermissionList from './modules/permission-list'
import Article from './modules/article'
import ArticleCreate from './modules/article-create'

/**
 * 私有路由表
 */
export const privateRoutes: RouteRecordRaw[] = [
  UserManage,
  RoleList,
  PermissionList,
  Article,
  ArticleCreate
]

/**
 * 公开路由表
 */
export const publicRoutes: RouteRecordRaw[] = [
  {
    path: '/',
    redirect: '/profile',
    component: Layout,
    children: [
      // 个人中心
      {
        path: '/profile',
        name: 'profile',
        meta: {
          title: 'profile',
          icon: 'el-icon-user'
        },
        component: () => import('@/views/profile/profile.vue')
      },
      {
        path: '/chart',
        name: 'chart',
        component: () => import('@/views/chart/chart.vue'),
        meta: {
          title: 'chart',
          icon: 'chart'
        }
      },
      // 401
      {
        path: '/401',
        name: '401',
        component: () => import('@/views/error-page/401.vue')
      },
      // 404
      {
        // path: '/404',
        path: '/:matchAll(.*)',
        name: '404',
        component: () => import('@/views/error-page/404.vue')
      }
    ]
  },
  {
    path: '/login',
    component: () => import('@/views/login/login.vue')
  }
]

const router = createRouter({
  history: createWebHistory(import.meta.env.BASE_URL),
  // 合并公有路由表和私有路由表
  routes: publicRoutes
})

/**
 * 初始化路由表
 */
export const resetRouter = () => {
  const userStore = useUserStore()
  if (userStore.userInfo?.permission?.menus) {
    const menus = userStore.userInfo.permission.menus
    menus.forEach((menu: string) => {
      // 根据路由对象的 name 字段移除动态添加的权限路由，也就是恢复路由的默认值
      router.removeRoute(menu)
    })
  }
}

export default router
