import HYRequest from './request'
import { BASE_URL, TIME_OUT, getSecret } from './config'
import { ElMessage } from 'element-plus'
import useUserStore from '@/stores/user'
import { storeToRefs } from 'pinia'
import { isLoginTimeout } from '@/utils'
import useAppStore from '@/stores/app'

const hyRequest = new HYRequest({
  baseURL: BASE_URL,
  timeout: TIME_OUT,
  interceptors: {
    requestSuccessFn(config) {
      config.headers = {
        ...config.headers,
        ...getSecret() // 携带验证字段数据
      } as any

      // 鉴权: 每一个请求都自动携带 token
      const userStore = useUserStore()
      const { token } = storeToRefs(userStore)
      if (token.value) {
        if (isLoginTimeout()) {
          // 登录超时( token 失效 ): 执行退出登录操作
          userStore.logout()
          ElMessage.error('登录超时')
          return Promise.reject(new Error('token 失效'))
        }

        config.headers.Authorization = 'Bearer ' + token.value
      }

      const appStore = useAppStore()
      config.headers['Accept-Language'] = appStore.language

      return config
    },
    requestFailureFn(error) {
      return Promise.reject(error)
    },
    responseSuccessFn(res: any) {
      if (res?.toString()?.includes('Error')) {
        return
      }
      const { success, message, data } = res
      if (success) {
        // 请求成功, 返回解析后的数据
        return data
      } else {
        // 失败 ( 请求成功, 业务失败 ), 消息提示
        // token 过期, 执行退出登录操作
        const result = res?.response
        if (result) {
          if (result.data?.code === 401) {
            useUserStore().logout()
          }
          if (result.status === 404) {
            return Promise.reject(res)
          } else {
            result.data?.message && ElMessage.error(result.data.message)
          }
        } else {
          message && ElMessage.error(message)
        }
        // 请求失败，直接将返回得到的数据
        return Promise.reject(res)
      }
    },
    responseFailureFn(error) {
      console.log('🚀 ~ file: index.ts:67 ~ responseFailureFn ~ error:', error)
      return Promise.reject(error)
    }
  }
})

export default hyRequest
