import { defineStore } from 'pinia'
import { THEME_DEFAULT_COLOR, THEME_STORE_CACHE_KEY } from '@/global/constants'
import scssVariables from '@/assets/styles/variables.module.scss'
import { generateColors } from '@/utils/theme'

type IStyleEl = HTMLStyleElement | null

interface IScssVars {
  [key: string]: string
}
interface IState {
  themeColor: string
  scssVariables: IScssVars
  styleEl: IStyleEl
}

const useThemeStore = defineStore('theme', {
  state: (): IState => ({
    scssVariables,
    themeColor: THEME_DEFAULT_COLOR, // 主题色
    styleEl: null
  }),
  getters: {
    // CSSModule 导出的变量, 该对象是只读的
    scssVars: (state) => ({
      // 首次加载页面时, 默认使用 scssVaribles 中的色值
      // 在用户选择了主题色之后, 才会覆盖默认色值
      ...state.scssVariables,
      ...generateColors(state.themeColor)
    })
  },
  actions: {
    /**
     * 设置网站主题色
     * @param color 主题色
     */
    setThemeColor(color: string) {
      this.themeColor = color
    },
    /**
     * 设置自定义 element-plus 时插入的 style 标签
     */
    setStyleEl(el: IStyleEl) {
      this.styleEl = el
    }
  },
  persist: {
    key: THEME_STORE_CACHE_KEY
  }
})

export default useThemeStore
