// ! 注意: 如果已经使用持久化插件处理了, 那么直接使用 Store 中的数据即可, 不需要再从浏览器缓存中读取
// 使用了持久化存储插件之后, 不需要再通过当前缓存工具类处理 Store 中的 state 数据了

/**
 * 封装 Storage 工具类 Cache
 * 可以选择 localStorage 或者 sessionStorage
 *
 * localStorage 和 sessionStorage 的区别
 *  localStorage 设置的数据会永久保存在浏览器中
 *  sessionStorage 设置的数据会在网页窗口被关闭后自动销毁
 */

import { isObject } from '.'

// 枚举类型
enum CacheType {
  Local,
  Session
}

class Cache {
  storage: Storage

  constructor(type: CacheType) {
    this.storage =
      type === CacheType.Local ? window.localStorage : window.sessionStorage
  }

  // 设置数据
  setCache(key: string, value: any) {
    // 1. 引用数据类型: [] {}
    if (value && isObject(value)) {
      return this.storage.setItem(key, JSON.stringify(value))
    }
    // 2. 基本数据类型
    this.storage.setItem(key, value)
  }

  // 获取数据
  getCache(key: string) {
    const value = this.storage.getItem(key)
    try {
      if (value) return JSON.parse(value)
    } catch (error) {
      return value
    }
  }

  // 移除指定数据
  removeCache(key: string) {
    this.storage.removeItem(key)
  }

  // 清空所有数据
  clearCache() {
    this.storage.clear()
  }
}

const localCache = new Cache(CacheType.Local)
const sessionCache = new Cache(CacheType.Session)

export { localCache, sessionCache }
